import csv
import io
from datetime import datetime
import pytz
import json

def generate_location_logs_csv(location_logs, patient_only=False):
    """Generate CSV for location logs
    
    Args:
        location_logs: List of location log objects
        patient_only: If True, only include patient data (flag == 0)
    """
    output = io.StringIO()
    writer = csv.writer(output)
    
    # Write headers
    writer.writerow([
        'Device Name',
        'Device MAC',
        'Room Name',
        'Patient Name',
        'Entry Time',
        'Exit Time',
        'Duration'
    ])
    
    time_zone = pytz.timezone('America/Los_Angeles')
    
    for log in location_logs:
        # Skip staff data if patient_only is True
        if patient_only and log.flag != 0:
            continue
            
        location_data = json.loads(log.location_log_arr)
        timeline_data = json.loads(log.location_timeline_arr)
        
        for entry in timeline_data:
            # Format entry and exit times
            start_time = ''
            end_time = ''
            if entry.get('start_time'):
                try:
                    # Try ISO format first (with 'T' separator)
                    start_time = datetime.fromisoformat(entry.get('start_time').replace('Z', '+00:00')).astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                except (ValueError, TypeError):
                    try:
                        # Fall back to standard format
                        start_time = datetime.strptime(entry.get('start_time'), '%Y-%m-%d %H:%M:%S').astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                    except (ValueError, TypeError):
                        # If all parsing fails, use the original string
                        start_time = entry.get('start_time', '')
            
            if entry.get('end_time'):
                try:
                    # Try ISO format first (with 'T' separator)
                    end_time = datetime.fromisoformat(entry.get('end_time').replace('Z', '+00:00')).astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                except (ValueError, TypeError):
                    try:
                        # Fall back to standard format
                        end_time = datetime.strptime(entry.get('end_time'), '%Y-%m-%d %H:%M:%S').astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                    except (ValueError, TypeError):
                        # If all parsing fails, use the original string
                        end_time = entry.get('end_time', '')
            
            writer.writerow([
                location_data.get('device_name', ''),
                location_data.get('dt_mac', ''),
                entry.get('room_name', ''),
                entry.get('name', ''),
                start_time,
                end_time,
                entry.get('duration', '')
            ])
    
    # Add UTF-8 BOM and return encoded string
    return '\ufeff' + output.getvalue()


def generate_alert_logs_csv(alert_logs, patient_only=False, site_id=None):
    """Generate CSV for alert logs
    
    Args:
        alert_logs: List of alert log objects
        patient_only: If True, only include patient data (patient_or_staff_id == 0)
        site_id: Optional site_id to filter alerts by site
    """
    output = io.StringIO()
    writer = csv.writer(output)
    
    # Write headers
    writer.writerow([
        'Patient Name',
        'Device Name',
        'Device MAC',
        'Room Name',
        'Zone Name',
        'Alert Type',
        'Creation Time',
        'Accepted By',
        'Accepted Time',
        'Closed Time',
        'Note',
    ])
    
    time_zone = pytz.timezone('America/Los_Angeles')
    
    for alert in alert_logs:
        # Skip staff data if patient_only is True
        if patient_only and alert.patient_or_staff_id != 0:
            continue
            
        # Skip if site_id is provided and doesn't match
        if site_id and alert.site_id != site_id:
            continue
            
        if alert.alert_log_array:
            alert_data = json.loads(alert.alert_log_array)
            
            # Get accepted details from alert_log_timeline
            accepted_time = ''
            closed_time = ''
            accepted_by = ''
            
            if alert.alert_log_timeline:
                timeline_logs = json.loads(alert.alert_log_timeline)
                # Reverse the timeline to get the latest status first
                timeline_logs = list(reversed(timeline_logs))
                
                # First find Accept status
                for log in timeline_logs:
                    if log.get('status') == 'Accept':
                        accepted_by = log.get('first_name', '')
                        created_date = log.get('created_at_date_details', '')
                        created_time = log.get('created_at_time_details', '')
                        if created_date and created_time:
                            try:
                                # Try ISO format first (with 'T' separator)
                                date_time_str = f"{created_date}T{created_time}"
                                dt = datetime.fromisoformat(date_time_str.replace('Z', '+00:00'))
                                accepted_time = dt.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                            except (ValueError, TypeError):
                                try:
                                    # Try standard format
                                    dt = datetime.strptime(f"{created_date} {created_time}", '%Y-%m-%d %H:%M:%S')
                                    accepted_time = dt.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                                except (ValueError, TypeError):
                                    try:
                                        # Try with comma separator
                                        dt = datetime.strptime(f"{created_date}, {created_time}", '%Y-%m-%d, %H:%M:%S')
                                        accepted_time = dt.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                                    except (ValueError, TypeError):
                                        accepted_time = f"{created_date}, {created_time}"
                        break
                
                # Then find Close status
                for log in timeline_logs:
                    if log.get('status') == 'Close':
                        created_date = log.get('created_at_date_details', '')
                        created_time = log.get('created_at_time_details', '')
                        if created_date and created_time:
                            try:
                                # Try ISO format first (with 'T' separator)
                                date_time_str = f"{created_date}T{created_time}"
                                dt = datetime.fromisoformat(date_time_str.replace('Z', '+00:00'))
                                closed_time = dt.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                            except (ValueError, TypeError):
                                try:
                                    # Try standard format
                                    dt = datetime.strptime(f"{created_date} {created_time}", '%Y-%m-%d %H:%M:%S')
                                    closed_time = dt.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                                except (ValueError, TypeError):
                                    try:
                                        # Try with comma separator
                                        dt = datetime.strptime(f"{created_date}, {created_time}", '%Y-%m-%d, %H:%M:%S')
                                        closed_time = dt.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
                                    except (ValueError, TypeError):
                                        closed_time = f"{created_date}, {created_time}"
                        break
            
            # Format creation time in the new format
            try:
                creation_time = alert.created_at.astimezone(time_zone).strftime('%d %b %Y, %I:%M:%S %p')
            except (ValueError, TypeError, AttributeError):
                creation_time = str(alert.created_at)
            
            writer.writerow([
                alert_data.get('patient_name', ''),
                alert_data.get('device_name', ''),
                alert_data.get('device_mac', ''),
                alert_data.get('room_name', ''),
                alert_data.get('zone_name', ''),
                alert_data.get('alert_type', ''),
                creation_time,
                accepted_by,
                accepted_time,
                closed_time,
                alert_data.get('note', '')
            ])
    
    return '\ufeff' + output.getvalue() 